/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSAppleEventManager;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSAppleEventDescriptor;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSAppleEventManager : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSAppleEventDescriptor appleEventForSuspensionID (objc.id suspensionID)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_appleEventForSuspensionID_1, suspensionID);
        return result !is null ? new NSAppleEventDescriptor(result) : null;
    }

    public NSAppleEventDescriptor currentAppleEvent ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_currentAppleEvent);
        return result !is null ? new NSAppleEventDescriptor(result) : null;
    }

    public NSAppleEventDescriptor currentReplyAppleEvent ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_currentReplyAppleEvent);
        return result !is null ? new NSAppleEventDescriptor(result) : null;
    }

    public short dispatchRawAppleEvent (objc.id theAppleEvent, objc.id theReply, objc.id handlerRefCon)
    {
        return cast(short) OS.objc_msgSend(this.id_, OS.sel_dispatchRawAppleEvent_1withRawReply_1handlerRefCon_1, theAppleEvent, theReply,
                handlerRefCon);
    }

    public void removeEventHandlerForEventClass (objc.id eventClass, objc.id eventID)
    {
        OS.objc_msgSend(this.id_, OS.sel_removeEventHandlerForEventClass_1andEventID_1, eventClass, eventID);
    }

    public NSAppleEventDescriptor replyAppleEventForSuspensionID (objc.id suspensionID)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_replyAppleEventForSuspensionID_1, suspensionID);
        return result !is null ? new NSAppleEventDescriptor(result) : null;
    }

    public void resumeWithSuspensionID (objc.id suspensionID)
    {
        OS.objc_msgSend(this.id_, OS.sel_resumeWithSuspensionID_1, suspensionID);
    }

    public void setCurrentAppleEventAndReplyEventWithSuspensionID (objc.id suspensionID)
    {
        OS.objc_msgSend(this.id_, OS.sel_setCurrentAppleEventAndReplyEventWithSuspensionID_1, suspensionID);
    }

    public void setEventHandler (id handler, objc.id handleEventSelector, objc.id eventClass, objc.id eventID)
    {
        OS.objc_msgSend(this.id_, OS.sel_setEventHandler_1andSelector_1forEventClass_1andEventID_1, handler !is null ? handler.id_ : null,
                handleEventSelector, eventClass, eventID);
    }

    public static NSAppleEventManager sharedAppleEventManager ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSAppleEventManager, OS.sel_sharedAppleEventManager);
        return result !is null ? new NSAppleEventManager(result) : null;
    }

    public objc.id suspendCurrentAppleEvent ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_suspendCurrentAppleEvent);
    }
}
