/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSArchiver;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSMutableData;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSArchiver : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public static bool archiveRootObject (id rootObject, NSString path)
    {
        return OS.objc_msgSend(OS.class_NSArchiver, OS.sel_archiveRootObject_1toFile_1, rootObject !is null ? rootObject.id_ : null,
                path !is null ? path.id_ : null) !is null;
    }

    public static NSData archivedDataWithRootObject (id rootObject)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSArchiver, OS.sel_archivedDataWithRootObject_1, rootObject !is null ? rootObject.id_ : null);
        return result !is null ? new NSData(result) : null;
    }

    public NSMutableData archiverData ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_archiverData);
        return result !is null ? new NSMutableData(result) : null;
    }

    public NSString classNameEncodedForTrueClassName (NSString trueName)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_classNameEncodedForTrueClassName_1, trueName !is null ? trueName.id_ : null);
        return result !is null ? new NSString(result) : null;
    }

    public void encodeClassName (NSString trueName, NSString inArchiveName)
    {
        OS.objc_msgSend(this.id_, OS.sel_encodeClassName_1intoClassName_1, trueName !is null ? trueName.id_ : null,
                inArchiveName !is null ? inArchiveName.id_ : null);
    }

    public void encodeConditionalObject (id object)
    {
        OS.objc_msgSend(this.id_, OS.sel_encodeConditionalObject_1, object !is null ? object.id_ : null);
    }

    public void encodeRootObject (id rootObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_encodeRootObject_1, rootObject !is null ? rootObject.id_ : null);
    }

    public NSArchiver initForWritingWithMutableData (NSMutableData mdata)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initForWritingWithMutableData_1, mdata !is null ? mdata.id_ : null);
        return result !is null ? this : null;
    }

    public void replaceObject (id object, id newObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_replaceObject_1withObject_1, object !is null ? object.id_ : null, newObject !is null ? newObject.id_ : null);
    }

}
