/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSDistributedLock;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSDate;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSDistributedLock : NSObject
{
    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void breakLock ()
    {
        OS.objc_msgSend(this.id_, OS.sel_breakLock);
    }

    public id initWithPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithPath_1, path !is null ? path.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public NSDate lockDate ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_lockDate);
        return result !is null ? new NSDate(result) : null;
    }

    public static NSDistributedLock lockWithPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSDistributedLock, OS.sel_lockWithPath_1, path !is null ? path.id_ : null);
        return result !is null ? new NSDistributedLock(result) : null;
    }

    public bool tryLock ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_tryLock) !is null;
    }

    public void unlock ()
    {
        OS.objc_msgSend(this.id_, OS.sel_unlock);
    }
}
