/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSFormCell;

import dwt.internal.cocoa.CGFloat;
import dwt.internal.cocoa.NSActionCell;
import dwt.internal.cocoa.NSAttributedString;
import dwt.internal.cocoa.NSFont;
import dwt.internal.cocoa.NSSize;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSText : NSTextAlignment, NSWritingDirection;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSFormCell : NSActionCell
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSAttributedString attributedTitle ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_attributedTitle);
        return result !is null ? new NSAttributedString(result) : null;
    }

    public bool isOpaque ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isOpaque) !is null;
    }

    public NSAttributedString placeholderAttributedString ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_placeholderAttributedString);
        return result !is null ? new NSAttributedString(result) : null;
    }

    public NSString placeholderString ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_placeholderString);
        return result !is null ? new NSString(result) : null;
    }

    public void setAttributedTitle (NSAttributedString obj)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAttributedTitle_1, obj !is null ? obj.id_ : null);
    }

    public void setPlaceholderAttributedString (NSAttributedString string)
    {
        OS.objc_msgSend(this.id_, OS.sel_setPlaceholderAttributedString_1, string !is null ? string.id_ : null);
    }

    public void setPlaceholderString (NSString string)
    {
        OS.objc_msgSend(this.id_, OS.sel_setPlaceholderString_1, string !is null ? string.id_ : null);
    }

    public void setTitle (NSString aString)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitle_1, aString !is null ? aString.id_ : null);
    }

    public void setTitleAlignment (NSTextAlignment mode)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleAlignment_1, mode);
    }

    public void setTitleBaseWritingDirection (NSWritingDirection writingDirection)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleBaseWritingDirection_1, writingDirection);
    }

    public void setTitleFont (NSFont fontObj)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleFont_1, fontObj !is null ? fontObj.id_ : null);
    }

    public void setTitleWidth (CGFloat width)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleWidth_1, width);
    }

    public void setTitleWithMnemonic (NSString stringWithAmpersand)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleWithMnemonic_1, stringWithAmpersand !is null ? stringWithAmpersand.id_ : null);
    }

    public NSString title ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_title);
        return result !is null ? new NSString(result) : null;
    }

    public NSTextAlignment titleAlignment ()
    {
        return cast(NSTextAlignment) OS.objc_msgSend(this.id_, OS.sel_titleAlignment);
    }

    public NSWritingDirection titleBaseWritingDirection ()
    {
        return cast(NSWritingDirection) OS.objc_msgSend(this.id_, OS.sel_titleBaseWritingDirection);
    }

    public NSFont titleFont ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_titleFont);
        return result !is null ? new NSFont(result) : null;
    }

    public CGFloat titleWidth ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id_, OS.sel_titleWidth);
    }

    public CGFloat titleWidth_ (NSSize aSize)
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id_, OS.sel_titleWidth_1, aSize);
    }

}
