/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSHTTPCookie;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSDate;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSURL;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSHTTPCookie : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSString comment ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_comment);
        return result !is null ? new NSString(result) : null;
    }

    public NSURL commentURL ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_commentURL);
        return result !is null ? new NSURL(result) : null;
    }

    public static id cookieWithProperties (NSDictionary properties)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHTTPCookie, OS.sel_cookieWithProperties_1, properties !is null ? properties.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public static NSArray cookiesWithResponseHeaderFields (NSDictionary headerFields, NSURL URL)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHTTPCookie, OS.sel_cookiesWithResponseHeaderFields_1forURL_1,
                headerFields !is null ? headerFields.id_ : null, URL !is null ? URL.id_ : null);
        return result !is null ? new NSArray(result) : null;
    }

    public NSString domain ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_domain);
        return result !is null ? new NSString(result) : null;
    }

    public NSDate expiresDate ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_expiresDate);
        return result !is null ? new NSDate(result) : null;
    }

    public id initWithProperties (NSDictionary properties)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithProperties_1, properties !is null ? properties.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public bool isSecure ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isSecure) !is null;
    }

    public bool isSessionOnly ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isSessionOnly) !is null;
    }

    public NSString name ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_name);
        return result !is null ? new NSString(result) : null;
    }

    public NSString path ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_path);
        return result !is null ? new NSString(result) : null;
    }

    public NSArray portList ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_portList);
        return result !is null ? new NSArray(result) : null;
    }

    public NSDictionary properties ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_properties);
        return result !is null ? new NSDictionary(result) : null;
    }

    public static NSDictionary requestHeaderFieldsWithCookies (NSArray cookies)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHTTPCookie, OS.sel_requestHeaderFieldsWithCookies_1, cookies !is null ? cookies.id_ : null);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSString value ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_value);
        return result !is null ? new NSString(result) : null;
    }

//public NSUInteger version() {
//  return OS.objc_msgSend(this.id_, OS.sel_version);
//}

}
