/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSInputStream;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSStream;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSInputStream : NSStream
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public bool getBuffer (ubyte** buffer, NSUInteger* len)
    {
        return OS.objc_msgSend(this.id_, OS.sel_getBuffer_1length_1, buffer, len) !is null;
    }

    public bool hasBytesAvailable ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_hasBytesAvailable) !is null;
    }

    public id initWithData (NSData data)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithData_1, data !is null ? data.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithFileAtPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithFileAtPath_1, path !is null ? path.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public static id inputStreamWithData (NSData data)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSInputStream, OS.sel_inputStreamWithData_1, data !is null ? data.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public static id inputStreamWithFileAtPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSInputStream, OS.sel_inputStreamWithFileAtPath_1, path !is null ? path.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public NSInteger read (ubyte* buffer, NSUInteger len)
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_read_1maxLength_1, buffer, len);
    }

}
