/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSMapTable;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSEnumerator;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSPointerFunctions;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSMapTable : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSUInteger count ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id_, OS.sel_count);
    }

    public NSDictionary dictionaryRepresentation ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_dictionaryRepresentation);
        return result !is null ? new NSDictionary(result) : null;
    }

    public id initWithKeyOptions (NSPointerFunctionsOptions keyOptions, NSPointerFunctionsOptions valueOptions, NSUInteger initialCapacity)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithKeyOptions_1valueOptions_1capacity_1, keyOptions, valueOptions, initialCapacity);
        return result !is null ? new id(result) : null;
    }

    public id initWithKeyPointerFunctions (NSPointerFunctions keyFunctions, NSPointerFunctions valueFunctions, NSUInteger initialCapacity)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithKeyPointerFunctions_1valuePointerFunctions_1capacity_1,
                keyFunctions !is null ? keyFunctions.id_ : null, valueFunctions !is null ? valueFunctions.id_ : null, initialCapacity);
        return result !is null ? new id(result) : null;
    }

    public NSEnumerator keyEnumerator ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_keyEnumerator);
        return result !is null ? new NSEnumerator(result) : null;
    }

    public NSPointerFunctions keyPointerFunctions ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_keyPointerFunctions);
        return result !is null ? new NSPointerFunctions(result) : null;
    }

    public static id mapTableWithKeyOptions (NSPointerFunctionsOptions keyOptions, NSPointerFunctionsOptions valueOptions)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMapTable, OS.sel_mapTableWithKeyOptions_1valueOptions_1, keyOptions, valueOptions);
        return result !is null ? new id(result) : null;
    }

    public static id mapTableWithStrongToStrongObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMapTable, OS.sel_mapTableWithStrongToStrongObjects);
        return result !is null ? new id(result) : null;
    }

    public static id mapTableWithStrongToWeakObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMapTable, OS.sel_mapTableWithStrongToWeakObjects);
        return result !is null ? new id(result) : null;
    }

    public static id mapTableWithWeakToStrongObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMapTable, OS.sel_mapTableWithWeakToStrongObjects);
        return result !is null ? new id(result) : null;
    }

    public static id mapTableWithWeakToWeakObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMapTable, OS.sel_mapTableWithWeakToWeakObjects);
        return result !is null ? new id(result) : null;
    }

    public NSEnumerator objectEnumerator ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_objectEnumerator);
        return result !is null ? new NSEnumerator(result) : null;
    }

    public id objectForKey (id aKey)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_objectForKey_1, aKey !is null ? aKey.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public void removeAllObjects ()
    {
        OS.objc_msgSend(this.id_, OS.sel_removeAllObjects);
    }

    public void removeObjectForKey (id aKey)
    {
        OS.objc_msgSend(this.id_, OS.sel_removeObjectForKey_1, aKey !is null ? aKey.id_ : null);
    }

    public void setObject (id anObject, id aKey)
    {
        OS.objc_msgSend(this.id_, OS.sel_setObject_1forKey_1, anObject !is null ? anObject.id_ : null, aKey !is null ? aKey.id_ : null);
    }

    public NSPointerFunctions valuePointerFunctions ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_valuePointerFunctions);
        return result !is null ? new NSPointerFunctions(result) : null;
    }

}
