/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSMenuItem;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSAttributedString;
import dwt.internal.cocoa.NSImage;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSMenu;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSView;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSMenuItem : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public objc.SEL action ()
    {
        return cast(objc.SEL) OS.objc_msgSend(this.id_, OS.sel_action);
    }

    public NSAttributedString attributedTitle ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_attributedTitle);
        return result !is null ? new NSAttributedString(result) : null;
    }

    public bool hasSubmenu ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_hasSubmenu) !is null;
    }

    public NSImage image ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_image);
        return result !is null ? new NSImage(result) : null;
    }

    public NSInteger indentationLevel ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_indentationLevel);
    }

    public NSMenuItem initWithTitle (NSString aString, objc.SEL aSelector, NSString charCode)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithTitle_1action_1keyEquivalent_1, aString !is null ? aString.id_ : null, aSelector,
                charCode !is null ? charCode.id_ : null);
        return result !is null ? this : null;
    }

    public bool isAlternate ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isAlternate) !is null;
    }

    public bool isEnabled ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isEnabled) !is null;
    }

    public bool isHidden ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isHidden) !is null;
    }

    public bool isHiddenOrHasHiddenAncestor ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isHiddenOrHasHiddenAncestor) !is null;
    }

    public bool isHighlighted ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isHighlighted) !is null;
    }

    public bool isSeparatorItem ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_isSeparatorItem) !is null;
    }

    public NSString keyEquivalent ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_keyEquivalent);
        return result !is null ? new NSString(result) : null;
    }

    public NSUInteger keyEquivalentModifierMask ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id_, OS.sel_keyEquivalentModifierMask);
    }

    public NSMenu menu ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_menu);
        return result !is null ? new NSMenu(result) : null;
    }

    public NSImage mixedStateImage ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_mixedStateImage);
        return result !is null ? new NSImage(result) : null;
    }

    public NSString mnemonic ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_mnemonic);
        return result !is null ? new NSString(result) : null;
    }

    public NSUInteger mnemonicLocation ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id_, OS.sel_mnemonicLocation);
    }

    public NSImage offStateImage ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_offStateImage);
        return result !is null ? new NSImage(result) : null;
    }

    public NSImage onStateImage ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_onStateImage);
        return result !is null ? new NSImage(result) : null;
    }

    public id representedObject ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_representedObject);
        return result !is null ? new id(result) : null;
    }

    public static NSMenuItem separatorItem ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMenuItem, OS.sel_separatorItem);
        return result !is null ? new NSMenuItem(result) : null;
    }

    public void setAction (objc.SEL aSelector)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAction_1, aSelector);
    }

    public void setAlternate (bool isAlternate)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAlternate_1, isAlternate);
    }

    public void setAttributedTitle (NSAttributedString string)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAttributedTitle_1, string !is null ? string.id_ : null);
    }

    public void setEnabled (bool flag)
    {
        OS.objc_msgSend(this.id_, OS.sel_setEnabled_1, flag);
    }

    public void setHidden (bool hidden)
    {
        OS.objc_msgSend(this.id_, OS.sel_setHidden_1, hidden);
    }

    public void setImage (NSImage menuImage)
    {
        OS.objc_msgSend(this.id_, OS.sel_setImage_1, menuImage !is null ? menuImage.id_ : null);
    }

    public void setIndentationLevel (NSInteger indentationLevel)
    {
        OS.objc_msgSend(this.id_, OS.sel_setIndentationLevel_1, indentationLevel);
    }

    public void setKeyEquivalent (NSString aKeyEquivalent)
    {
        OS.objc_msgSend(this.id_, OS.sel_setKeyEquivalent_1, aKeyEquivalent !is null ? aKeyEquivalent.id_ : null);
    }

    public void setKeyEquivalentModifierMask (NSUInteger mask)
    {
        OS.objc_msgSend(this.id_, OS.sel_setKeyEquivalentModifierMask_1, mask);
    }

    public void setMenu (NSMenu menu)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMenu_1, menu !is null ? menu.id_ : null);
    }

    public void setMixedStateImage (NSImage image)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMixedStateImage_1, image !is null ? image.id_ : null);
    }

    public void setMnemonicLocation (NSUInteger location)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMnemonicLocation_1, location);
    }

    public void setOffStateImage (NSImage image)
    {
        OS.objc_msgSend(this.id_, OS.sel_setOffStateImage_1, image !is null ? image.id_ : null);
    }

    public void setOnStateImage (NSImage image)
    {
        OS.objc_msgSend(this.id_, OS.sel_setOnStateImage_1, image !is null ? image.id_ : null);
    }

    public void setRepresentedObject (id anObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_setRepresentedObject_1, anObject !is null ? anObject.id_ : null);
    }

    public void setState (NSInteger state)
    {
        OS.objc_msgSend(this.id_, OS.sel_setState_1, state);
    }

    public void setSubmenu (NSMenu submenu)
    {
        OS.objc_msgSend(this.id_, OS.sel_setSubmenu_1, submenu !is null ? submenu.id_ : null);
    }

    public void setTag (NSInteger anInt)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTag_1, anInt);
    }

    public void setTarget (id anObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTarget_1, anObject !is null ? anObject.id_ : null);
    }

    public void setTitle (NSString aString)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitle_1, aString !is null ? aString.id_ : null);
    }

    public void setTitleWithMnemonic (NSString stringWithAmpersand)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTitleWithMnemonic_1, stringWithAmpersand !is null ? stringWithAmpersand.id_ : null);
    }

    public void setToolTip (NSString toolTip)
    {
        OS.objc_msgSend(this.id_, OS.sel_setToolTip_1, toolTip !is null ? toolTip.id_ : null);
    }

    public static void setUsesUserKeyEquivalents (bool flag)
    {
        OS.objc_msgSend(OS.class_NSMenuItem, OS.sel_setUsesUserKeyEquivalents_1, flag);
    }

    public void setView (NSView view)
    {
        OS.objc_msgSend(this.id_, OS.sel_setView_1, view !is null ? view.id_ : null);
    }

    public NSInteger state ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_state);
    }

    public NSMenu submenu ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_submenu);
        return result !is null ? new NSMenu(result) : null;
    }

    public NSInteger tag ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_tag);
    }

    public id target ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_target);
        return result !is null ? new id(result) : null;
    }

    public NSString title ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_title);
        return result !is null ? new NSString(result) : null;
    }

    public NSString toolTip ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_toolTip);
        return result !is null ? new NSString(result) : null;
    }

    public NSString userKeyEquivalent ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_userKeyEquivalent);
        return result !is null ? new NSString(result) : null;
    }

    public static bool usesUserKeyEquivalents ()
    {
        return OS.objc_msgSend(OS.class_NSMenuItem, OS.sel_usesUserKeyEquivalents) !is null;
    }

    public NSView view ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_view);
        return result !is null ? new NSView(result) : null;
    }

}
