/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSURLConnection;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSRunLoop;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSURLRequest;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSURLConnection : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public static bool canHandleRequest (NSURLRequest request)
    {
        return OS.objc_msgSend(OS.class_NSURLConnection, OS.sel_canHandleRequest_1, request !is null ? request.id_ : null) !is null;
    }

    public void cancel ()
    {
        OS.objc_msgSend(this.id_, OS.sel_cancel);
    }

    public static NSURLConnection connectionWithRequest (NSURLRequest request, id delegatee)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSURLConnection, OS.sel_connectionWithRequest_1delegate_1, request !is null ? request.id_ : null,
                delegatee !is null ? delegatee.id_ : null);
        return result !is null ? new NSURLConnection(result) : null;
    }

    public id initWithRequest_delegate_ (NSURLRequest request, id delegatee)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithRequest_1delegate_1, request !is null ? request.id_ : null,
                delegatee !is null ? delegatee.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithRequest_delegate_startImmediately_ (NSURLRequest request, id delegatee, bool startImmediately)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithRequest_1delegate_1startImmediately_1, request !is null ? request.id_ : null,
                delegatee !is null ? delegatee.id_ : null, startImmediately);
        return result !is null ? new id(result) : null;
    }

    public void scheduleInRunLoop (NSRunLoop aRunLoop, NSString mode)
    {
        OS.objc_msgSend(this.id_, OS.sel_scheduleInRunLoop_1forMode_1, aRunLoop !is null ? aRunLoop.id_ : null, mode !is null ? mode.id_ : null);
    }

    public static NSData sendSynchronousRequest (NSURLRequest request, /*NSURLResponse** */objc.id** response, /*NSError** */objc.id** error)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSURLConnection, OS.sel_sendSynchronousRequest_1returningResponse_1error_1,
                request !is null ? request.id_ : null, response, error);
        return result !is null ? new NSData(result) : null;
    }

    public void start ()
    {
        OS.objc_msgSend(this.id_, OS.sel_start);
    }

    public void unscheduleFromRunLoop (NSRunLoop aRunLoop, NSString mode)
    {
        OS.objc_msgSend(this.id_, OS.sel_unscheduleFromRunLoop_1forMode_1, aRunLoop !is null ? aRunLoop.id_ : null, mode !is null ? mode.id_ : null);
    }

}
