/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSURLCredentialStorage;

import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSURLCredential;
import dwt.internal.cocoa.NSURLProtectionSpace;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSURLCredentialStorage : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSDictionary allCredentials ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_allCredentials);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSDictionary credentialsForProtectionSpace (NSURLProtectionSpace space)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_credentialsForProtectionSpace_1, space !is null ? space.id_ : null);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSURLCredential defaultCredentialForProtectionSpace (NSURLProtectionSpace space)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_defaultCredentialForProtectionSpace_1, space !is null ? space.id_ : null);
        return result !is null ? new NSURLCredential(result) : null;
    }

    public void removeCredential (NSURLCredential credential, NSURLProtectionSpace space)
    {
        OS.objc_msgSend(this.id_, OS.sel_removeCredential_1forProtectionSpace_1, credential !is null ? credential.id_ : null,
                space !is null ? space.id_ : null);
    }

    public void setCredential (NSURLCredential credential, NSURLProtectionSpace space)
    {
        OS.objc_msgSend(this.id_, OS.sel_setCredential_1forProtectionSpace_1, credential !is null ? credential.id_ : null,
                space !is null ? space.id_ : null);
    }

    public void setDefaultCredential (NSURLCredential credential, NSURLProtectionSpace space)
    {
        OS.objc_msgSend(this.id_, OS.sel_setDefaultCredential_1forProtectionSpace_1, credential !is null ? credential.id_ : null,
                space !is null ? space.id_ : null);
    }

    public static NSURLCredentialStorage sharedCredentialStorage ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSURLCredentialStorage, OS.sel_sharedCredentialStorage);
        return result !is null ? new NSURLCredentialStorage(result) : null;
    }

}
